@@ -3,7 +3,6 @@ import logging
 import re
 import socket
 import struct
-import thread
 import threading
 from urllib import splitport
 import zlib
@@ -14,7 +13,7 @@ from bmemcached.exceptions import AuthenticationNotSupported, InvalidCredentials
 logger = logging.getLogger(__name__)
 
 
-class Protocol(object):
+class Protocol(threading.local):
     """
     This class is used by Client class to communicate with server.
     """
@@ -61,23 +60,10 @@ class Protocol(object):
 
     COMPRESSION_THRESHOLD = 128
 
-    _thread_instances = {}
-
-    def __new__(cls, *args, **kw):
-        """
-        Server singleton
-        """
-        instance_key = '%s-%s-%s' % (thread.get_ident(), str(args), str(kw))
-        if instance_key not in cls._thread_instances:
-            cls._thread_instances[instance_key] = super(Protocol, cls).__new__(
-                cls, *args, **kw)
-
-        return cls._thread_instances[instance_key]
-
-    def __init__(self, server, username=None, password=None):
+    def __init__(self, server, username=None, password=None, compression=None):
         self.server = server
         self.authenticated = False
-        self._lock = threading.Lock()
+        self.compression = zlib if compression is None else compression
 
         if server.startswith('/'):
             self.connection = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
@@ -91,17 +77,6 @@ class Protocol(object):
         if username and password:
             self.authenticate(username, password)
 
-    def lock(f):
-        def _f(self, *args, **kwargs):
-            self._lock.acquire()
-            try:
-                rv = f(self, *args, **kwargs)
-            finally:
-                self._lock.release()
-
-            return rv
-        return _f
-
     def split_host_port(self, server):
         """
         Return (host, port) from server.
@@ -234,7 +209,7 @@ class Protocol(object):
             value = dumps(value)
 
         if len(value) > self.COMPRESSION_THRESHOLD:
-            value = zlib.compress(value)
+            value = self.compression.compress(value)
             flags |= self.FLAGS['compressed']
 
         return flags, value
@@ -251,7 +226,7 @@ class Protocol(object):
         :rtype: basestring|int
         """
         if flags & self.FLAGS['compressed']:  # pragma: no branch
-            value = zlib.decompress(value)
+            value = self.compression.decompress(value)
 
         if flags & self.FLAGS['integer']:
             return int(value)
